#!/usr/bin/python3

# This script shall be used during update from 1.0.1 to 1.1.0 version
# of vpnportal to migrate the database.
#
# CS service must be stopped before this script is started.
#

from   functions   import g_file_cs_db, g_file_cs_settings
from   database    import SettingsDB
from   db_migrator import DBMigrator
from   ipaddress   import IPv4Network

####################################################################
# Surrounds the text with quotes.
def add_quotes(text):
  return '\'' + text + '\''

####################################################################
# Assigns permanent IPs to roadwarriors in DB.
def assign_ovpn_ips_to_rws(dbm):
  count           = dbm.count_entries('roadwarrior_table')
  network, mask   = get_rw_network_settings(g_file_cs_settings)
  hosts = list(IPv4Network(network + '/' + mask).hosts())
  hosts = hosts[1:] # First IP in range belongs to server
  if len(hosts) < count:
    raise RuntimeError("The network for Standalone Clients is too small. \
                        There are not enough addresses for " + count + " VPN Standalone Clients.")
  next_free_ip = hosts[count]
  hosts = hosts[:count]
  dbm.update_col_values("system_table", "next_unused_rw_ip", [add_quotes(str(next_free_ip))])
  dbm.update_col_values("device_table", "ovpn_ip", [add_quotes(str(ip)) for ip in hosts], from_back=True)

####################################################################
# Returns network and MASK for RW OVPN server.
def get_rw_network_settings(settings_db_path):
  s = SettingsDB(settings_db_path)
  network = s.get_value('ROADWARRIOR_SERVER1_NETWORK')
  mask    = s.get_value('ROADWARRIOR_SERVER2_MASK')
  return (network, mask)

####################################################################
####################################################################

print('Migrating database from 1.0.1 to 1.1.0')

OP_SET_LAN   = "0"
OP_CREATE_RW = "9"

dbm = DBMigrator(g_file_cs_db)

# Version 1.1.* added a new table: 'router_table'. Various columns got redistributed
# between device_table, router_table and roadwarrior_table.  
dbm.derive_new_table('device_table', 'router_table', ['device_id', 'dev_certificate', 'pending_count', 'serial'])

dbm.add_column('router_table', 'proxy_enabled_lan', 'integer DEFAULT 0 NOT NULL', '0', pos=1)
dbm.add_column('router_table', 'internet_access',   'integer DEFAULT 1 NOT NULL', '1', pos=1)

# Rename 'serial' to 'udid'.
dbm.edit_column('router_table', 'serial', 'udid', 'text UNIQUE NOT NULL')

dbm.del_column('device_table', 'ovpn_cn')
dbm.del_column('device_table', 'dev_certificate')
dbm.del_column('device_table', 'pending_count')
dbm.del_column('device_table', 'serial')
dbm.del_column('device_table', 'quarantine')
dbm.del_column('device_table', 'quarantine_reason')

# Pos parameter is used so that in the schema we keep device_id on the 
# first row and only after that there will be device_type.
# 2 = device type "Router" (while "1" is device type "Roadwarrior")
dbm.add_column('device_table', 'device_type',          'integer NOT NULL', '2', pos=1)
dbm.add_column('device_table', 'label',                'text', '', pos=1)
dbm.add_column('device_table', 'disabled',             'integer NOT NULL DEFAULT 0', '0', pos=1)
dbm.add_column('device_table', 'proxy_enabled_direct', 'integer NOT NULL DEFAULT 0', '0', pos=1)
dbm.add_column('device_table', 'ovpn_cert_serial',     'text UNIQUE', '', pos=1)

# To satisfy the UNIQUE specifier of the 'ovpn_cert_serial'
# column, we temporarily insert device_ids there. They will 
# be replaced with proper values in a different script.
dbm.copy_column_values('device_table', 'device_table', 'device_id', 'ovpn_cert_serial')

dbm.add_column( 'system_table', 'next_unused_rw_ip', 'text', '') 
dbm.edit_column('system_table', 'next_unused_ovpn_ip', 'next_unused_router_ip')

# Each roadwarrior must also have a row in device_table. 
dbm.copy_entries('roadwarrior_table', 'device_table', {"online":"$online", "name":"$rw_name", "validated":"1", "device_type":"1"})

dbm.del_column('roadwarrior_table', 'rw_id')
dbm.del_column('roadwarrior_table', 'online')
dbm.del_column('roadwarrior_table', 'rw_name')
dbm.add_column('roadwarrior_table', 'device_id', 'integer', value='')

freed_ips_table='''create table freed_rw_ips_table (
                      ovpn_ip text PRIMARY KEY
                   );
                '''
prevalidation_table='''create table prevalidation_table (
                        udid   text PRIMARY KEY,
                        serial text NOT NULL,
                        mac    text NOT NULL,
                        imei   text NOT NULL,
                        name   text
                      );
                    '''
pending_internet_a_table='''create table pending_internet_access_table (
                              trans_id         integer PRIMARY KEY,
                              device_id        integer NOT NULL,
                              internet_access  integer NOT NULL,
                              error_code       integer DEFAULT 0
                            );
                         '''
connlog_table='''create table connlog_table (
                  connlog_id      integer PRIMARY KEY,
                  device_id       integer NOT NULL,
                  conn_event_time integer NOT NULL,
                  conn_event_type integer NOT NULL,
                  bytes_sent      integer,
                  bytes_received  integer,
                  uptime          integer
                );
             '''
dbm.create_table(freed_ips_table)
dbm.create_table(prevalidation_table)
dbm.create_table(pending_internet_a_table)
dbm.create_table(connlog_table)

dbm.edit_trigger_body("decrement_pending_count3", "device_table", "router_table")
dbm.edit_trigger_body("decrement_pending_count2", "device_table", "router_table")
dbm.edit_trigger_body("decrement_pending_count1", "device_table", "router_table")

dbm.edit_trigger_body("increment_pending_count3", "device_table", "router_table")
dbm.edit_trigger_body("increment_pending_count2", "device_table", "router_table")
dbm.edit_trigger_body("increment_pending_count1", "device_table", "router_table")

trigger_4_inc='''CREATE TRIGGER increment_pending_count4
 AFTER INSERT ON pending_internet_access_table
 BEGIN
    UPDATE router_table SET pending_count = pending_count + 1 WHERE device_id = NEW.device_id;
 END;
'''
trigger_4_dec='''CREATE TRIGGER decrement_pending_count4
 AFTER DELETE ON pending_internet_access_table
 BEGIN
    UPDATE router_table SET pending_count = pending_count - 1 WHERE device_id = OLD.device_id;
 END;
'''
dbm.create_trigger(trigger_4_inc)
dbm.create_trigger(trigger_4_dec)

dbm.edit_column('roadwarrior_table', 'expiration_days', 'expiration_date')

dbm.commit()

# Information about expiration of roadwarrior client certificates in 1.0.* was 
# stored as days until expiration.
# In 1.1.0 it is stored as the seconds from start of epoch. This change affected contents of 
# transactions that create roadwarriors. For this reason we will delete all such preexisting
# transactions now and rename a column.
dbm.db.run("delete from transaction_table where operation_type=\""+ OP_CREATE_RW +"\";")

# Instead of SET_LAN we have SET_LANS transaction in 1.1.0.
dbm.db.run("delete from pending_lan_table;")
dbm.db.run("delete from transaction_table where operation_type=\""+ OP_SET_LAN +"\";")

# Rename a table. 
dbm.db.run("ALTER TABLE freed_ovpn_ips_table RENAME TO freed_router_ips_table;")

dbm.reload()

# Copy last N device_ids from device_table to roadwarrior_table. The reason why this
# cannot be done before the first commit() is called is that device_ids of the new 
# records (which we created previously with 'copy_entries()') are assigned automaticly
# by sqlite and therefore are not known until the commit is called.
#    
dbm.copy_column_values('device_table', 'roadwarrior_table', src_col='device_id', dst_col='device_id', from_back=True)

# Now that device_ids have been copied from device_table, we can set the column 
# to be "PRIMARY KEY". This could not have been done earlier, because PRIMARY KEY
# cannot be empty.
dbm.edit_column('roadwarrior_table', 'device_id', 'device_id', "integer PRIMARY KEY")

# In 1.0.* the OVPN IP of roadwarriors was assigned randomly. In 
# 1.1.0 their IP is permanently stored in DB. So we now need to 
# generate IPs for all existing roadwarriors and write them into
# the database.
assign_ovpn_ips_to_rws(dbm)

dbm.commit()



