#!/usr/bin/python3

# The purpose of this script is to create new certificates and
# keys for roadwarriors (because in the 1.0.0 version their CN
# was equal to 'rw_name' column, but in 1.1.0 version their CN  
# is equal to their device_id).
#
# After creating the new keys, the script then updates downloadable
# config files of all routers with the new credentials.
#
# This script must be run with 'sudo' privileges and only AFTER 
# the previous certificates were revoked/deleted and the database 
# migrated.

import time
import os

from functions import g_file_cs_db, g_dir_rw_cli_certs
from database  import Database

##############################################################
# Creates an openssl config in 'out_file'.
def create_openssl_config(out_file, cn, org, location, country, email, expiration_days):
  s = '[ca]\n'\
      'default_ca = ca_default\n'\
      '[req]\n'\
      'prompt = no\n'\
      'distinguished_name = req_distinguished_name\n'\
      '[req_distinguished_name]\n'\
      'CN = '+cn+'\n'\
      'C  = '+country+'\n'\
      'O  = '+org+'\n'\
      'L  = '+location+'\n'\
      'emailAddress = '+email+'\n'\
      '[ca_default]\n'\
      'certificate      = /opt/vpnportal/customer_server/certificates/ca/ca.crt\n'\
      'private_key      = /opt/vpnportal/customer_server/certificates/ca/ca.pem\n'\
      'database         = /opt/vpnportal/customer_server/certificates/ca/index.txt\n'\
      'serial           = /opt/vpnportal/customer_server/certificates/ca/serial.txt\n'\
      'new_certs_dir    = "/tmp/"\n'\
      'default_days     = '+ str(expiration_days) +'\n'\
      'policy           = policy_match\n'\
      'default_md       = sha256\n'\
      'x509_extensions  = v3_ca\n'\
      '[v3_ca]\n'\
      'authorityKeyIdentifier = keyid:always\n'\
      'extendedKeyUsage       = critical, clientAuth\n'\
      'keyUsage               = digitalSignature, keyEncipherment\n'\
      'basicConstraints       = critical, CA:FALSE\n'\
      '[policy_match]\n'\
      'commonName          = supplied\n'\
      'countryName         = optional\n'\
      'stateOrProvinceName = optional\n'\
      'organizationName    = optional\n'\
      'organizationalUnitName = optional\n'\
      'localityName        = optional\n'\
      'emailAddress        = optional\n'
  f = open(out_file, "w")
  f.write(s)

##############################################################
# Returns the number of days between 'now' and 'date_in_seconds'.
def get_days_until(date_in_seconds):
  today = time.time()
  return round((date_in_seconds - today) / (3600 * 24))

##############################################################
# Generates a key and cert for the given device. The openssl
# configuration file must be ready and its path be passed as 
# 'tmp_cfg_path' param.
def create_credentials(device_id, tmp_cfg_path):
  key_out  = g_dir_rw_cli_certs + "/" + device_id + ".pem"
  cert_out = g_dir_rw_cli_certs + "/" + device_id + ".crt"
  csr_out  = g_dir_rw_cli_certs + "/" + device_id + ".csr"
  # Generate key
  os.system("/usr/bin/openssl genpkey -algorithm EC -pkeyopt ec_paramgen_curve:P-256 -pkeyopt ec_param_enc:named_curve -out " + key_out)
  # Generate CSR
  os.system("/usr/bin/openssl req -new -key "+key_out+" -out "+csr_out+" -config "+tmp_cfg_path)
  # Generate certificate
  os.system("/usr/bin/openssl ca -out "+cert_out+" -in "+csr_out+" -batch -notext -extensions v3_ca -config "+tmp_cfg_path)
  # Clean up temporary files
  os.remove(tmp_cfg_path)
  os.remove(csr_out)
  # Load the new key and cert
  with open(key_out, 'r') as f:
    key = f.read()
  with open(cert_out, 'r') as f:
    cert = f.read()  
  return cert, key

##############################################################
# Replaces key and cert in standa's downloadable config file.
def update_downloadable_config(config_path, cert, key):
  with open(config_path, 'r') as f:
    downloadable_config = f.read()
  # Edit the downloadable config in memory
  start = downloadable_config.find("<cert>")
  end   = downloadable_config.find("</cert>")+8
  downloadable_config = downloadable_config[:start] + downloadable_config[end:]
  start = downloadable_config.find("<key>")
  end   = downloadable_config.find("</key>")+7
  downloadable_config = downloadable_config[:start] + downloadable_config[end:]
  downloadable_config += "<cert>\n"+cert+"\n</cert>\n"
  downloadable_config += "<key>\n"+key+"\n</key>\n"
  # Write the modified downloadable config back to file
  with open(config_path, 'w') as f:
    f.write(downloadable_config)

##############################################################
##############################################################

if os.geteuid() != 0:
  print('This script must be run with sudo!')  
  exit(1)

db = Database(g_file_cs_db)

tmp_cfg_path = "/tmp/openssl.cfg"

# Read info about standas.
rows = db.run('select device_id, organization, location, country, rw_email, expiration_date, config_file_path from roadwarrior_table;')

# For each Standalone client do:
for row in rows:
  # Get days remaining until the standa expires.
  expiration_days = get_days_until(row[5])
  
  # Get his device_id.
  device_id       = str(row[0])
  
  # Prepare an openssl configuration file for generation of new key pair
  create_openssl_config(tmp_cfg_path, device_id, row[1], row[2], row[3], row[4], expiration_days)
  
  # Run the commands that will create the key and cert on disk. Also return the
  # contents of the cert and key. 
  cert, key = create_credentials(device_id, tmp_cfg_path)
  
  # Remove the old key and cert from downloadable config and store the 
  # new new key and cert there.
  update_downloadable_config(row[6], cert, key)














































