<?php

namespace AppBundle\Api;

use Monolog\Logger;

/**
 * Class Device.
 */
class Device
{
    const pending_operation_status_pending = 0;
    const pending_operation_status_failed = 1;

    /**
     * @var Logger
     */
    protected $log;

    /**
     * @var \PhpClientApi
     */
    protected $phpClientApi;

    /**
     * Device constructor.
     *
     * @param $phpClientApi
     * @param Logger $log
     */
    public function __construct($phpClientApi, Logger $log)
    {
        $this->phpClientApi = $phpClientApi;
        $this->log = $log;
    }

    /**
     * @param $network_id
     * @param $start
     * @param $length
     * @param array $order
     * @param array $filter
     *
     * @return array
     */
    public function getDevicesNotInNetwork($network_id, $start, $length, $order = array(), array $filter = array())
    {
        try {
            $devList = $this->phpClientApi->get_devices_not_in_network($network_id, $start, $length, $filter, $order);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            $devList = array();
        }

        return $devList;
    }

    public function getAllRouters($start, $length, array $order = array(), array $filter = array())
    {
        $aOrder = array();

        foreach ($order as $col => $order) {
            $aOrder[] = array($col, $order);
        }

        try {
            $routerList = $this->phpClientApi->get_all_routers($start, $length, $filter, $aOrder);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            $devList = array();
        }

        return $routerList;
    }

    /**
     * @param $start
     * @param $length
     * @param array $order
     * @param array $filter
     *
     * @return array
     */
    public function getAllDevices($start, $length, array $order = array(), array $filter = array())
    {
        $aOrder = array();

        foreach ($order as $col => $order) {
            $aOrder[] = array($col, $order);
        }

        try {
            $devList = $this->phpClientApi->get_all_devices($start, $length, $filter, $aOrder);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            $devList = array();
        }

        return $devList;
    }

    /**
     * @param $deviceId
     *
     * @return mixed
     */
    public function getDevicePendingLans($deviceId)
    {
        try {
            return $this->phpClientApi->get_device_pending_lans((int) $deviceId);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return array();
        }
    }

    /**
     * @param $deviceId
     *
     * @return array
     */
    public function getLans($deviceId)
    {
        try {
            return $this->phpClientApi->get_lans((int) $deviceId);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return array();
        }
    }

    /**
     * @param $lans
     * @param $deviceId
     *
     * @return bool|string
     */
    public function setLans($lans, $device_id)
    {
        $lan_configs = array();

        foreach ($lans as $lan) {
            $lan_cfg = array();
            $lan_cfg['mode'] = strval($lan->getMode());
            $lan_cfg['lan_iface'] = $lan->getName();

            switch ($lan->getMode()) {
                case 0:

                    //Ignore mode
                    break;

                case 1:

                    //Private
                    $lan_cfg['lan_iface_ip'] = $lan->getIfaceIp();
                    $lan_cfg['lan_netmask'] = $lan->getNetmask();
                    $lan_cfg['dhcp_enabled'] = ((true === $lan->getDhcpEnabled()) ? '1' : '0');
                    $lan_cfg['dhcp_from'] = $lan->getDhcpFrom();
                    $lan_cfg['dhcp_to'] = $lan->getDhcpTo();
                    break;

                case 2:

                    //Public
                    $lan_cfg['lan_iface_ip'] = $lan->getIfaceIp();
                    $lan_cfg['lan_netmask'] = $lan->getNetmask();
                    $lan_cfg['dhcp_enabled'] = ((true === $lan->getDhcpEnabled()) ? '1' : '0');
                    $lan_cfg['dhcp_from'] = $lan->getDhcpFrom();
                    $lan_cfg['dhcp_to'] = $lan->getDhcpTo();
                    break;

                case 3:

                    //1 to 1 NAT
                    $lan_cfg['lan_iface_ip'] = $lan->getIfaceIp();
                    $lan_cfg['lan_netmask'] = $lan->getNetmask();
                    $lan_cfg['dhcp_enabled'] = ((true === $lan->getDhcpEnabled()) ? '1' : '0');
                    $lan_cfg['dhcp_from'] = $lan->getDhcpFrom();
                    $lan_cfg['dhcp_to'] = $lan->getDhcpTo();
                    $lan_cfg['virtual_ip'] = $lan->getVirtualIp();
                    $lan_cfg['virtual_mask'] = $lan->getVirtualNetmask();
                    break;
            }

            array_push($lan_configs, $lan_cfg);
        }

        try {
            $errors = $this->phpClientApi->verify_lan_settings($device_id, $lan_configs);
            if (!empty($errors)) {
                return $errors;
            }

            $this->phpClientApi->set_lans($device_id, $lan_configs);

            return null;
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return $e->getMessage();
        }
    }

    /**
     * @param $deviceProxy
     * @param $deviceId
     *
     * @return bool|string
     */
    public function setDeviceProxy($device_id, $deviceProxy)
    {
        try {
            return $this->phpClientApi->set_direct_proxy_enabled($device_id, $deviceProxy);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return $e->getMessage();
        }
    }

    /**
     * @param $lanProxy
     * @param $deviceId
     *
     * @return bool|string
     */
    public function setLanProxy($device_id, $lanProxy)
    {
        try {
            return $this->phpClientApi->set_lan_proxy_enabled($device_id, $lanProxy);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return $e->getMessage();
        }
    }

    /**
     * @param $deviceId
     *
     * @return bool
     */
    public function mutateLinks($deviceId)
    {
        try {
            return $this->phpClientApi->mutate_proxy_links((int) $deviceId);
            //return true;
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return false;
        }
    }

    /**
     * @param $deviceId
     *
     * @return array
     */
    public function getInternetAccess($deviceId)
    {
        try {
            return $this->phpClientApi->get_internet_access((int) $deviceId);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return array();
        }
    }

    /**
     * @param $internetAccess
     * @param $deviceId
     *
     * @return bool|string
     */
    public function setInternetAccess($device_id, $internetAccess)
    {
        try {
            return $this->phpClientApi->set_internet_access($device_id, $internetAccess);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return $e->getMessage();
        }
    }

    /**
     * @param $deviceId
     * @param $interface
     * @param $data
     *
     * @return bool|string
     */
    public function validatePrivateLanSettings($deviceId, $interface, $data)
    {
        try {
            $this->phpClientApi->verify_private_lan_settings($deviceId, $interface, $data);

            return true;
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return $e->getMessage();
        }
    }

    /**
     * @param $deviceId
     * @param $interface
     * @param $data
     *
     * @return bool|string
     */
    public function validatePublicLanSettings($deviceId, $interface, $data)
    {
        try {
            $this->phpClientApi->verify_public_lan_settings($deviceId, $interface, $data);

            return true;
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return $e->getMessage();
        }
    }

    /**
     * @param $deviceId
     * @param $interface
     * @param $data
     *
     * @return bool|string
     */
    public function validate1to1NatLanSettings($deviceId, $interface, $data)
    {
        try {
            $this->phpClientApi->verify_o2o_nat_lan_settings($deviceId, $interface, $data);

            return true;
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return $e->getMessage();
        }
    }

    /**
     * @param int   $start
     * @param int   $length
     * @param array $order
     * @param array $filter
     *
     * @return array
     */

    /**
     * @param $deviceId
     *
     * @return bool|string
     */
    public function rescanLans($device_id)
    {
        try {
            return $this->phpClientApi->rescan_lan_ifaces($device_id);

            //return NULL;
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return $e->getMessage();
        }
    }

    public function getAllNetworks($start = 0, $length = 1000, $order = array('network_id' => 'asc'), $filter = array())
    {
        $aOrder = array();

        foreach ($order as $col => $order) {
            $aOrder[] = array($col, $order);
        }

        try {
            $netList = $this->phpClientApi->get_all_networks($start, $length, $filter, $aOrder);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            $netList = array();
        }

        return $netList;
    }

    /**
     * @param null $filter
     *
     * @return mixed
     */
    public function getCountOfAllDevices($filter = null)
    {
        if (null == $filter) {
            $filter = array();
        }

        try {
            return $this->phpClientApi->get_all_devices_count($filter);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return 0;
        }
    }

    /**
     * @param null $filter
     *
     * @return mixed
     */
    public function getCountOfAllRouters($filter = null)
    {
        if (null == $filter) {
            $filter = array();
        }

        try {
            return $this->phpClientApi->get_all_routers_count($filter);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return 0;
        }
    }

    /**
     * @param $networkId
     * @param null $filter
     *
     * @return mixed
     */
    public function getCountOfDevicesInNetwork($networkId, $filter = null)
    {
        if (null == $filter) {
            $filter = array();
        }

        try {
            return $this->phpClientApi->get_devices_in_network_count($networkId, $filter);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return 0;
        }
    }

    /**
     * @param null $filter
     *
     * @return mixed
     */
    public function getCountOfAllNetworks($filter = null)
    {
        if (null == $filter) {
            $filter = array();
        }

        try {
            return $this->phpClientApi->get_all_networks_count($filter);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return 0;
        }
    }

    /**
     * @param $deviceId
     *
     * @return mixed
     */
    public function getDeviceById($deviceId)
    {
        try {
            return $this->phpClientApi->get_device_by_id((int) $deviceId);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return array();
        }
    }

    /**
     * @param $deviceId
     *
     * @return mixed
     */
    public function getRouterById($deviceId)
    {
        try {
            return $this->phpClientApi->get_router_by_id((int) $deviceId);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return array();
        }
    }

    /**
     * @param $deviceId
     *
     * @return mixed
     */
    public function getDeviceConfigById($deviceId)
    {
        try {
            return $this->phpClientApi->get_device_config((int) $deviceId);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return array();
        }
    }

    /**
     * @param $networkId
     *
     * @return mixed
     */
    public function getNetworkById($networkId)
    {
        try {
            return $this->phpClientApi->get_network_by_id((int) $networkId);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return array();
        }
    }

    /**
     * @param $name
     *
     * @return bool|string
     */
    public function addNetwork($name)
    {
        try {
            //return $this->phpClientApi->add_network($name);
            $this->phpClientApi->add_network($name);

            return true;
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            //return false;
            return $e->getMessage();
        }
    }

    /**
     * @param $networkId
     * @param $name
     *
     * @return mixed
     */
    public function updateNetwork($networkId, $name)
    {
        try {
            //return $this->phpClientApi->update_network((int) $networkId, $name);
            $this->phpClientApi->update_network((int) $networkId, $name);

            return true;
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            //return false;
            return $e->getMessage();
        }
    }

    /**
     * @param $deviceId
     *
     * @return mixed
     */
    public function getNetworksByDeviceId($deviceId)
    {
        try {
            return $this->phpClientApi->get_networks_by_device_id((int) $deviceId);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return array();
        }
    }

    /**
     * @param $networkId
     * @param $start
     * @param $length
     * @param array $order
     * @param array $filter
     *
     * @return mixed
     */
    public function getDevicesByNetworkId($networkId, $start, $length, $order = array('name' => 'asc'), $filter = array())
    {
        $aOrder = array();

        foreach ($order as $col => $order) {
            $aOrder[] = array($col, $order);
        }

        try {
            return $this->phpClientApi->get_devices_by_network_id((int) $networkId, $start, $length, $filter, $aOrder);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return array();
        }
    }

    /**
     * @param null $filter
     *
     * @return mixed
     */
    public function getCountOfDevicesInNetworks($networkId, $filter = null)
    {
        if (null == $filter) {
            $filter = array();
        }

        try {
            return $this->phpClientApi->get_all_networks_count($networkId, $filter);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return 0;
        }
    }

    /**
     * @param $deviceId
     *
     * @return mixed
     *
     * @throws \Exception
     */
    public function validateDevice($deviceId)
    {
        try {
            return $this->phpClientApi->validate_device((int) $deviceId);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            throw new \Exception($e->getMessage());
        }
    }

    /**
     * @param array $deviceIds
     *
     * @return mixed
     *
     * @throws \Exception
     */
    public function validateDevices(array $deviceIds)
    {
        try {
            return $this->phpClientApi->validate_devices($deviceIds);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            throw new \Exception($e->getMessage());
        }
    }

    /**
     * @param $deviceId
     * @param $name
     *
     * @return bool
     */
    public function setDeviceName($deviceId, $name)
    {
        try {
            return $this->phpClientApi->set_name((int) $deviceId, $name);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return false;
        }
    }

    /**
     * @param $deviceId
     * @param $label
     *
     * @return bool
     */
    public function setDeviceLabel($deviceId, $label)
    {
        try {
            return $this->phpClientApi->set_label((int) $deviceId, $label);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return false;
        }
    }

    /**
     * @param $deviceId
     * @param $ip
     * @param $netMask
     * @param $interface
     *
     * @return mixed
     */
    public function setLanDevice($deviceId, $ip, $netMask, $interface)
    {
        try {
            if (is_null($ip)) {
                $ip = '';
            }

            if (is_null($netMask)) {
                $netMask = '';
            }

            $this->phpClientApi->set_lan((int) $deviceId, $ip, $netMask, $interface);

            return true;
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return $e->getMessage();
        }
    }

    /**
     * @param $deviceId
     * @param $networkId
     *
     * @return mixed
     */
    public function joinDeviceNetwork($deviceId, $networkId)
    {
        try {
            $deviceIds = array($deviceId);

            return $this->phpClientApi->join_network((int) $networkId, $deviceIds);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return false;
        }
    }

    /**
     * @param array $deviceIds
     * @param $networkId
     *
     * @return bool
     */
    public function joinDevicesNetwork(array $deviceIds, $networkId)
    {
        try {
            return $this->phpClientApi->join_network((int) $networkId, $deviceIds);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return false;
        }
    }

    /**
     * @param $deviceId
     * @param $networkId
     *
     * @return mixed
     */
    public function leaveDeviceNetwork($deviceId, $networkId)
    {
        try {
            $deviceIds = array($deviceId);

            return $this->phpClientApi->leave_network((int) $networkId, $deviceIds);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return false;
        }
    }

    /**
     * @param $deviceId
     *
     * @return bool
     */
    public function deleteDevice($deviceId)
    {
        try {
            return $this->phpClientApi->delete_device((int) $deviceId);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return false;
        }
    }

    /**
     * @param array $deviceIds
     *
     * @return bool
     */
    public function deleteDevices($deviceIds)
    {
        try {
            return $this->phpClientApi->delete_devices($deviceIds);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return false;
        }
    }

    /**
     * @param array $deviceIds
     *
     * @return bool
     */
    public function disableDevices($deviceIds)
    {
        try {
            return $this->phpClientApi->disable_devices($deviceIds);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return false;
        }
    }

    /**
     * @param array $deviceIds
     *
     * @return bool
     */
    public function enableDevices($deviceIds)
    {
        try {
            return $this->phpClientApi->enable_devices($deviceIds);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return false;
        }
    }

    /**
     * @param $networkId
     *
     * @return bool
     */
    public function deleteNetwork($networkId)
    {
        try {
            return $this->phpClientApi->delete_network((int) $networkId);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return false;
        }
    }

    /**
     * @param $networkIds
     *
     * @return bool
     */
    public function deleteNetworks($networkIds)
    {
        try {
            return $this->phpClientApi->delete_networks($networkIds);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return false;
        }
    }

    /**
     * @param $deviceId
     *
     * @return mixed
     */
    public function getDeviceStats($deviceId)
    {
        try {
            return $this->phpClientApi->get_device_stats((int) $deviceId);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return array();
        }
    }

    /**
     * @param $file
     *
     * @return bool
     */
    public function setPrevalidationList($file)
    {
        try {
            $this->phpClientApi->set_preval_list($file);
            return "";
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);
            return $e->getMessage();
        }
    }

    /**
     * @return array
     */
    public function getPrevalidationList($start, $length, array $order = array(), array $filter = array())
    {
        $aOrder = array();

        foreach ($order as $col => $order) {
            $aOrder[] = array($col, $order);
        }

        try {
            $prevalidationList = $this->phpClientApi->get_preval_list($start, $length, $filter, $aOrder);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            $prevalidationList = array();
        }

        return $prevalidationList;
    }

    /**
     * @param null $filter
     *
     * @return mixed
     */
    public function getCountOfPrevalidated($filter = null)
    {
        if (null == $filter) {
            $filter = array();
        }

        try {
            return $this->phpClientApi->get_preval_count($filter);
        } catch (\Exception $e) {
            $this->logError(__FUNCTION__, func_get_args(), $e);

            return 0;
        }
    }

    /**
     * @param $functionName
     * @param array           $arguments
     * @param \Exception|null $exception
     */
    private function logError($functionName, array $arguments, \Exception $exception = null)
    {
        try {
            $ref = new \ReflectionMethod($this, $functionName);
            $functionParameters = [];

            foreach ($ref->getParameters() as $key => $currentParameter) {
                $functionParameters[$currentParameter->getName()] = $arguments[$key];
            }

            $this->log->addError('Exception '.((null !== $exception) ? '"'.$exception->getMessage().'"' : '').' on '.$functionName.', parameters:'.print_r(serialize($functionParameters), true));
        } catch (\Exception $e) {
            $this->log->addError('Exception '.((null !== $exception) ? '"'.$exception->getMessage().'"' : '').' on '.$functionName);
        }
    }
}
