<?php

namespace AppBundle\Api;

use Monolog\Logger;

/**
 * Class FwRules
 * @package AppBundle\Api
 */
class FwRules{

    const RELATED_POLICY_ACCEPT = 0;
    const RELATED_POLICY_DROP = 1;

    const TRAFFIC_DIRECTION_INCOMING = 1;
    const TRAFFIC_DIRECTION_OUTGOING = 2;

    /**
     * @var \PhpClientApi
     */
    protected $phpClientApi;

    /**
     * @var Logger
     */
    protected $log;

    /**
     * FwRules constructor.
     * @param $phpClientApi
     * @param Logger $log
     */
    function __construct($phpClientApi, Logger $log) {

        $this->phpClientApi = $phpClientApi;
        $this->log = $log;

    }

    /**
     * @param $deviceId
     * @param array $filter
     * @return array
     */
    public function getDeviceFwRules($deviceId, $filter = array()) {

        try{

            $fwRules = $this->phpClientApi->fw_get_dev_filters($deviceId, $filter);

            $whitelist = array('ports', 'src_ips', 'dst_ips', 'protocol' , 'related_policy', 'traffic_direction', 'enabled');

            if(!empty($fwRules)){

                foreach ($fwRules as $key => $fwRule){

                    $fwRules[$key] = array_intersect_key( $fwRule, array_flip( $whitelist ) );

                }

            }else{

                $fwRules = array();

            }

        }catch (\Exception $e){

            $this->logError(__FUNCTION__, func_get_args(), $e);

            $fwRules = array();

        }

        return $fwRules;

    }

    /**
     * @param $deviceId
     * @param $direction
     * @return |null
     */
    public function getDeviceFwPolicy($deviceId, $direction) {

        try{

            $fwPolicy = $this->phpClientApi->fw_get_dev_filter_policy($deviceId, $direction);

        }catch (\Exception $e){

            $this->logError(__FUNCTION__, func_get_args(), $e);

            $fwPolicy = null;

        }

        return $fwPolicy;

    }

    /**
     * @param $deviceId
     * @param $trafficDirection IN = 1, OUT = 2
     * @param $policy ACCEPT=0, DROP=1
     */
    public function setDeviceFwPolicy($deviceId, $trafficDirection, $policy) {

        try{

            $this->phpClientApi->fw_set_dev_filter_policy($deviceId, $trafficDirection, $policy);

            return true;

        }catch (\Exception $e){

            $this->logError(__FUNCTION__, func_get_args(), $e);

            return false;

        }

    }

    /**
     * @param $deviceId
     * @param array $filter
     * @return array
     */
    public function getNetworkFwRules($networkId, $filter = array()) {

        try{

            $fwRules = $this->phpClientApi->fw_get_net_filters($networkId, $filter);

            $whitelist = array('ports', 'protocol' , 'related_policy', 'enabled');

            if(!empty($fwRules)){

                foreach ($fwRules as $key => $fwRule){

                    $fwRules[$key] = array_intersect_key( $fwRule, array_flip( $whitelist ) );

                }

            }else{

                $fwRules = array();

            }

        }catch (\Exception $e){

            $this->logError(__FUNCTION__, func_get_args(), $e);

            $fwRules = array();

        }

        return $fwRules;

    }

    /**
     * @param $networkId
     * @return |null
     */
    public function getNetworkFwPolicy($networkId) {

        try{

            $fwPolicy = $this->phpClientApi->fw_get_net_filter_policy($networkId);

        }catch (\Exception $e){

            $this->logError(__FUNCTION__, func_get_args(), $e);

            $fwPolicy = null;

        }

        return $fwPolicy;

    }

    /**
     * @param $networkId
     * @param $policy ACCEPT=0, DROP=1
     */
    public function setNetworkFwPolicy($networkId, $policy) {

        try{

            $this->phpClientApi->fw_set_net_filter_policy($networkId, $policy);

            return true;

        }catch (\Exception $e){

            $this->logError(__FUNCTION__, func_get_args(), $e);

            return false;

        }

    }

    /**
     * @param $deviceId
     * @param array $rules
     * @return bool
     */
    public function addDeviceFwRules($deviceId, array $rules){

        try{

            $this->phpClientApi->fw_setup_dev_filters($deviceId, $rules);

            return true;

        }catch (\Exception $e){

            $this->logError(__FUNCTION__, func_get_args(), $e);

            throw $e;

        }

    }

    /**
     * @param $networkId
     * @param array $rules
     * @return bool
     */
    public function addNetworkFwRules($networkId, array $rules){

        try{

            $this->phpClientApi->fw_setup_net_filters($networkId, $rules);

            return true;

        }catch (\Exception $e){

            $this->logError(__FUNCTION__, func_get_args(), $e);

            throw $e;

        }

    }

    /**
     * @param $functionName
     * @param array $arguments
     * @param \Exception|null $exception
     */
    private function logError($functionName, array $arguments, \Exception $exception = null){

        try{

            $ref = new \ReflectionMethod($this, $functionName);
            $functionParameters = [];

            foreach($ref->getParameters() as $key => $currentParameter) {

                $functionParameters[$currentParameter->getName()] = $arguments[$key];

            }

            $this->log->addError('Exception ' . (($exception !== null) ? '"' .$exception->getMessage(). '"' : '').' on '. $functionName .', parameters:' . print_r(serialize($functionParameters), true));

        }catch (\Exception $e){

            $this->log->addError('Exception ' . (($exception !== null) ? '"' .$exception->getMessage(). '"' : '').' on '. $functionName);

        }

    }

}